@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2015 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -32,7 +32,7 @@ import types
 
 from Mailman import mm_cfg
 from Mailman import Utils
-from Mailman.i18n import _
+from Mailman.i18n import _, get_translation
 
 from Mailman.CSRFcheck import csrf_token
 
@@ -319,6 +319,8 @@ class Document(Container):
                           'content="text/html; charset=%s">' % charset)
             if self.title:
                 output.append('%s<TITLE>%s</TITLE>' % (tab, self.title))
+            if mm_cfg.WEB_HEAD_ADD:
+                output.append(mm_cfg.WEB_HEAD_ADD)
             output.append('%s</HEAD>' % tab)
             quals = []
             # Default link colors
@@ -443,6 +445,7 @@ class InputObj:
         self.kws = kws
 
     def Format(self, indent=0):
+        charset = get_translation().charset() or 'us-ascii'
         output = ['<INPUT name="%s" type="%s" value="%s"' %
                   (self.name, self.type, self.value)]
         for item in self.kws.items():
@@ -450,7 +453,10 @@ class InputObj:
         if self.checked:
             output.append('CHECKED')
         output.append('>')
-        return SPACE.join(output)
+        ret = SPACE.join(output)
+        if self.type == 'TEXT' and isinstance(ret, unicode):
+            ret = ret.encode(charset, 'replace')
+        return ret
 
 
 class SubmitButton(InputObj):
@@ -488,6 +494,7 @@ class TextArea:
         self.readonly = readonly
 
     def Format(self, indent=0):
+        charset = get_translation().charset() or 'us-ascii'
         output = '<TEXTAREA NAME=%s' % self.name
         if self.rows:
             output += ' ROWS=%s' % self.rows
@@ -498,6 +505,8 @@ class TextArea:
         if self.readonly:
             output += ' READONLY'
         output += '>%s</TEXTAREA>' % self.text
+        if isinstance(output, unicode):
+            output = output.encode(charset, 'replace')
         return output
 
 class FileUpload(InputObj):
@@ -542,7 +551,9 @@ class WidgetArray:
                                   self.button_names,
                                   self.values):
             ischecked = (self.ischecked(i))
-            item = self.Widget(self.name, value, ischecked).Format() + name
+            item = ('<label>' +
+                    self.Widget(self.name, value, ischecked).Format() +
+                    name + '</label>')
             items.append(item)
             if not self.horizontal:
                 t.AddRow(items)
